@@ -41,17 +41,22 @@ const (
 type Resources interface {
 	All([]ResourceType, AllOpts) ([]Resource, error)
 	Delete(Resource) error
-	Exists(Resource) (bool, error)
+	Exists(Resource, ExistsOpts) (bool, error)
 	Get(Resource) (Resource, error)
 	Patch(Resource, types.PatchType, []byte) (Resource, error)
 	Update(Resource) (Resource, error)
 	Create(resource Resource) (Resource, error)
 }
 
+type ExistsOpts struct {
+	SameUID bool
+}
+
 type ResourcesImpl struct {
 	resourceTypes             ResourceTypes
 	coreClient                kubernetes.Interface
 	dynamicClient             dynamic.Interface
+	mutedDynamicClient        dynamic.Interface
 	fallbackAllowedNamespaces []string
 
 	assumedAllowedNamespacesMemoLock sync.Mutex
@@ -61,12 +66,14 @@ type ResourcesImpl struct {
 }
 
 func NewResourcesImpl(resourceTypes ResourceTypes, coreClient kubernetes.Interface,
-	dynamicClient dynamic.Interface, fallbackAllowedNamespaces []string, logger logger.Logger) *ResourcesImpl {
+	dynamicClient dynamic.Interface, mutedDynamicClient dynamic.Interface,
+	fallbackAllowedNamespaces []string, logger logger.Logger) *ResourcesImpl {
 
 	return &ResourcesImpl{
 		resourceTypes:             resourceTypes,
 		coreClient:                coreClient,
 		dynamicClient:             dynamicClient,
+		mutedDynamicClient:        mutedDynamicClient,
 		fallbackAllowedNamespaces: fallbackAllowedNamespaces,
 		logger:                    logger.NewPrefixed("Resources"),
 	}
@@ -101,7 +108,7 @@ func (c *ResourcesImpl) All(resTypes []ResourceType, opts AllOpts) ([]Resource,
 			var list *unstructured.UnstructuredList
 			var err error
 
-			client := c.dynamicClient.Resource(resType.GroupVersionResource)
+			client := c.mutedDynamicClient.Resource(resType.GroupVersionResource)
 
 			err = util.Retry2(time.Second, 5*time.Second, c.isServerRescaleErr, func() error {
 				if resType.Namespaced() {
@@ -222,7 +229,7 @@ func (c *ResourcesImpl) Create(resource Resource) (Resource, error) {
 		c.logger.Debug("create resource %s\n%s\n", resource.Description(), bs)
 	}
 
-	resClient, resType, err := c.resourceClient(resource)
+	resClient, resType, err := c.resourceClient(resource, resourceClientOpts{Warnings: true})
 	if err != nil {
 		return nil, err
 	}
@@ -249,7 +256,7 @@ func (c *ResourcesImpl) Update(resource Resource) (Resource, error) {
 		c.logger.Debug("update resource %s\n%s\n", resource.Description(), bs)
 	}
 
-	resClient, resType, err := c.resourceClient(resource)
+	resClient, resType, err := c.resourceClient(resource, resourceClientOpts{Warnings: true})
 	if err != nil {
 		return nil, err
 	}
@@ -273,7 +280,7 @@ func (c *ResourcesImpl) Patch(resource Resource, patchType types.PatchType, data
 		defer func() { c.logger.Debug("patch %s", time.Now().UTC().Sub(t1)) }()
 	}
 
-	resClient, resType, err := c.resourceClient(resource)
+	resClient, resType, err := c.resourceClient(resource, resourceClientOpts{Warnings: true})
 	if err != nil {
 		return nil, err
 	}
@@ -302,7 +309,7 @@ func (c *ResourcesImpl) Delete(resource Resource) error {
 		return nil
 	}
 
-	resClient, resType, err := c.resourceClient(resource)
+	resClient, resType, err := c.resourceClient(resource, resourceClientOpts{Warnings: true})
 	if err != nil {
 		return err
 	}
@@ -343,7 +350,7 @@ func (c *ResourcesImpl) Get(resource Resource) (Resource, error) {
 		defer func() { c.logger.Debug("get %s", time.Now().UTC().Sub(t1)) }()
 	}
 
-	resClient, resType, err := c.resourceClient(resource)
+	resClient, resType, err := c.resourceClient(resource, resourceClientOpts{Warnings: false})
 	if err != nil {
 		return nil, err
 	}
@@ -362,13 +369,13 @@ func (c *ResourcesImpl) Get(resource Resource) (Resource, error) {
 	return NewResourceUnstructured(*item, resType), nil
 }
 
-func (c *ResourcesImpl) Exists(resource Resource) (bool, error) {
+func (c *ResourcesImpl) Exists(resource Resource, existsOpts ExistsOpts) (bool, error) {
 	if resourcesDebug {
 		t1 := time.Now().UTC()
 		defer func() { c.logger.Debug("exists %s", time.Now().UTC().Sub(t1)) }()
 	}
 
-	resClient, _, err := c.resourceClient(resource)
+	resClient, _, err := c.resourceClient(resource, resourceClientOpts{Warnings: false})
 	if err != nil {
 		// Assume if type is not known to the API server
 		// then such resource cannot exist on the server
@@ -381,7 +388,7 @@ func (c *ResourcesImpl) Exists(resource Resource) (bool, error) {
 	var found bool
 
 	err = util.Retry(time.Second, time.Minute, func() (bool, error) {
-		_, err = resClient.Get(context.TODO(), resource.Name(), metav1.GetOptions{})
+		fetchedRes, err := resClient.Get(context.TODO(), resource.Name(), metav1.GetOptions{})
 		if err != nil {
 			if errors.IsNotFound(err) {
 				found = false
@@ -398,7 +405,17 @@ func (c *ResourcesImpl) Exists(resource Resource) (bool, error) {
 			isDone := errors.IsForbidden(err)
 			// TODO sometimes metav1.StatusReasonUnknown is returned (empty string)
 			// might be related to deletion of mutating webhook
-			return isDone, c.resourceErr(err, "Checking existance of", resource)
+			return isDone, c.resourceErr(err, "Checking existence of", resource)
+		}
+
+		// Check if we have to compare the UID's also to confirm if it is same resource.
+		if existsOpts.SameUID {
+			if fetchedRes != nil {
+				if string(fetchedRes.GetUID()) != resource.UID() {
+					found = false
+					return true, nil
+				}
+			}
 		}
 
 		found = true
@@ -409,7 +426,7 @@ func (c *ResourcesImpl) Exists(resource Resource) (bool, error) {
 }
 
 var (
-	// Error example: Checking existance of resource podmetrics/knative-ingressgateway-646d475cbb-c82qb (metrics.k8s.io/v1beta1)
+	// Error example: Checking existence of resource podmetrics/knative-ingressgateway-646d475cbb-c82qb (metrics.k8s.io/v1beta1)
 	//   namespace: istio-system: Error while getting pod knative-ingressgateway-646d475cbb-c82qb:
 	//   pod "knative-ingressgateway-646d475cbb-c82qb" not found (reason: )
 	// Note that it says pod is not found even though we were checking on podmetrics.
@@ -419,7 +436,7 @@ var (
 
 func (c *ResourcesImpl) isPodMetrics(resource Resource, err error) bool {
 	// Abnormal error case. Get/Delete on PodMetrics may fail
-	// without NotFound reason due to its dependence on Pod existance
+	// without NotFound reason due to its dependence on Pod existence
 	if resource.Kind() == "PodMetrics" && resource.APIGroup() == "metrics.k8s.io" {
 		if podMetricsNotFoundErrCheck.MatchString(err.Error()) {
 			return true
@@ -429,7 +446,7 @@ func (c *ResourcesImpl) isPodMetrics(resource Resource, err error) bool {
 }
 
 func (c *ResourcesImpl) isGeneralRetryableErr(err error) bool {
-	return IsResourceChangeBlockedErr(err) || c.isServerRescaleErr(err) ||
+	return IsResourceChangeBlockedErr(err) || c.isServerRescaleErr(err) || c.isEtcdRetryableError(err) ||
 		c.isResourceQuotaConflict(err) || c.isInternalFailure(err) || errors.IsTooManyRequests(err)
 }
 
@@ -472,13 +489,24 @@ func (c *ResourcesImpl) resourceErr(err error, action string, resource Resource)
 	return resourcePlainErr{err, action, resource}
 }
 
-func (c *ResourcesImpl) resourceClient(resource Resource) (dynamic.ResourceInterface, ResourceType, error) {
+type resourceClientOpts struct {
+	Warnings bool
+}
+
+func (c *ResourcesImpl) resourceClient(resource Resource, opts resourceClientOpts) (dynamic.ResourceInterface, ResourceType, error) {
 	resType, err := c.resourceTypes.Find(resource)
 	if err != nil {
 		return nil, ResourceType{}, err
 	}
 
-	return c.dynamicClient.Resource(resType.GroupVersionResource).Namespace(resource.Namespace()), resType, nil
+	var dynamicClient dynamic.Interface
+	if opts.Warnings {
+		dynamicClient = c.dynamicClient
+	} else {
+		dynamicClient = c.mutedDynamicClient
+	}
+
+	return dynamicClient.Resource(resType.GroupVersionResource).Namespace(resource.Namespace()), resType, nil
 }
 
 func (c *ResourcesImpl) assumedAllowedNamespaces() ([]string, error) {
@@ -515,6 +543,10 @@ var (
 	//   Post https://cert-manager-webhook.cert-manager.svc:443/convert?timeout=30s:
 	//   x509: certificate signed by unknown authority (reason: )
 	conversionWebhookErrCheck = regexp.MustCompile("conversion webhook for (.+) failed:")
+
+	// Matches retryable etcdserver errors
+	// Comprehensive list of errors at : https://github.com/etcd-io/etcd/blob/main/server/etcdserver/errors.go
+	etcdserverRetryableErrCheck = regexp.MustCompile("etcdserver:(.+)(leader changed|timed out)")
 )
 
 func IsResourceChangeBlockedErr(err error) bool {
@@ -532,6 +564,12 @@ func IsResourceChangeBlockedErr(err error) bool {
 	}
 }
 
+// Retries retryable errors thrown by etcd server.
+// Addresses : https://github.com/vmware-tanzu/carvel-kapp/issues/106
+func (c *ResourcesImpl) isEtcdRetryableError(err error) bool {
+	return etcdserverRetryableErrCheck.MatchString(err.Error())
+}
+
 type AllOpts struct {
 	ListOpts *metav1.ListOptions
 }
