@@ -24,9 +24,11 @@ func TestWarningsFlag(t *testing.T) {
 	logger := Logger{}
 	kapp := Kapp{t, env.Namespace, env.KappBinaryPath, Logger{}}
 	crdName := "test-no-warnings-crd"
-	crName := "test-no-warnings-cr"
+	crName1 := "test-no-warnings-cr1"
+	crName2 := "test-no-warnings-cr2"
 	cleanUp := func() {
-		kapp.Run([]string{"delete", "-a", crName})
+		kapp.Run([]string{"delete", "-a", crName1})
+		kapp.Run([]string{"delete", "-a", crName2})
 		kapp.Run([]string{"delete", "-a", crdName})
 	}
 
@@ -77,22 +79,68 @@ spec:
 		kapp.RunWithOpts([]string{"deploy", "-f", "-", "-a", crdName},
 			RunOpts{StdinReader: strings.NewReader(yaml)})
 	})
-	logger.Section("deploying with --warnings flag", func() {
+
+	logger.Section("deploying without --warnings flag", func() {
 		yaml := strings.Replace(crYaml, "<cr-name>", "cr-1", 1)
-		out, _ := kapp.RunWithOpts([]string{"deploy", "-f", "-", "-a", crName, "--warnings=true"},
+		out, _ := kapp.RunWithOpts([]string{"deploy", "-f", "-", "-a", crName1},
 			RunOpts{StdinReader: strings.NewReader(yaml)})
 
-		if !strings.Contains(out, customWarning) {
-			t.Fatalf("Expected warning %s, but didn't get", customWarning)
+		expectedOutput := `
+Changes
+
+Namespace  Name  Kind     Conds.  Age  Op      Op st.  Wait to    Rs  Ri  
+kapp-test  cr-1  CronTab  -       -    create  -       reconcile  -   -  
+
+Op:      1 create, 0 delete, 0 update, 0 noop
+Wait to: 1 reconcile, 0 delete, 0 noop
+
+<replaced>: ---- applying 1 changes [0/1 done] ----
+Warning: <custom-warning>
+<replaced>: create crontab/cr-1 (stable.example.com/v1alpha1) namespace: kapp-test
+<replaced>: ---- waiting on 1 changes [0/1 done] ----
+<replaced>: ok: reconcile crontab/cr-1 (stable.example.com/v1alpha1) namespace: kapp-test
+<replaced>: ---- applying complete [1/1 done] ----
+<replaced>: ---- waiting complete [1/1 done] ----
+
+Succeeded`
+
+		out = strings.TrimSpace(replaceTarget(replaceSpaces(replaceTs(out))))
+		expectedOutput = strings.Replace(expectedOutput, "<custom-warning>", customWarning, 1)
+		expectedOutput = strings.TrimSpace(replaceSpaces(expectedOutput))
+
+		if expectedOutput != out {
+			t.Fatalf("Expected output with warning >>%s<<, but got >>%s<<\n", expectedOutput, out)
 		}
 	})
-	logger.Section("deploying without --warnings flag", func() {
+
+	logger.Section("deploying with --warnings flag", func() {
 		yaml := strings.Replace(crYaml, "<cr-name>", "cr-2", 1)
-		out, _ := kapp.RunWithOpts([]string{"deploy", "-f", "-", "-a", crName},
+
+		out, _ := kapp.RunWithOpts([]string{"deploy", "-f", "-", "-a", crName2, "--warnings=false"},
 			RunOpts{StdinReader: strings.NewReader(yaml)})
 
-		if strings.Contains(out, customWarning) {
-			t.Fatalf("Expected no warning, but got %s", customWarning)
+		expectedOutput := `
+Changes
+
+Namespace  Name  Kind     Conds.  Age  Op      Op st.  Wait to    Rs  Ri  
+kapp-test  cr-2  CronTab  -       -    create  -       reconcile  -   -  
+
+Op:      1 create, 0 delete, 0 update, 0 noop
+Wait to: 1 reconcile, 0 delete, 0 noop
+
+<replaced>: ---- applying 1 changes [0/1 done] ----
+<replaced>: create crontab/cr-2 (stable.example.com/v1alpha1) namespace: kapp-test
+<replaced>: ---- waiting on 1 changes [0/1 done] ----
+<replaced>: ok: reconcile crontab/cr-2 (stable.example.com/v1alpha1) namespace: kapp-test
+<replaced>: ---- applying complete [1/1 done] ----
+<replaced>: ---- waiting complete [1/1 done] ----
+
+Succeeded`
+
+		out = strings.TrimSpace(replaceTarget(replaceSpaces(replaceTs(out))))
+		expectedOutput = strings.TrimSpace(replaceSpaces(expectedOutput))
+		if expectedOutput != out {
+			t.Fatalf("Expected output without warning >>%s<< but got >>%s<<\n", expectedOutput, out)
 		}
 	})
 }
